@@ -21,6 +21,11 @@
 
 endpoints <-
 function(x,on='months',k=1) {
+
+  if(k < 1) {
+    stop("'k' must be > 0")
+  }
+
   if(timeBased(x)) {
     NR <- length(x)
     x <- xts(, order.by=x)
@@ -41,6 +46,15 @@ function(x,on='months',k=1) {
   if(on %in% c('years','quarters','months','weeks','days'))
     posixltindex <- as.POSIXlt(.POSIXct(.index(x)),tz=tzone(x))
 
+  include_last <- function(x, k) {
+    len <- length(x)
+    i <- seq(1L ,len, k)
+    if(i[length(i)] != len) {
+      i <- c(i, len)
+    }
+    ep[i]
+  }
+
   switch(on,
     "years" = {
       as.integer(c(0, which(diff(posixltindex$year %/% k + 1) != 0), NR))
@@ -49,43 +63,45 @@ function(x,on='months',k=1) {
       ixyear <- posixltindex$year * 100L + 190000L
       ixqtr <- ixyear + posixltindex$mon %/% 3L + 1L
       ep <- c(0L, which(diff(ixqtr) != 0L), NR)
-      if(k > 1)
-        ep[seq(1,length(ep),k)]
-      else ep
+      if(k > 1) {
+        ep <- include_last(ep, k)
+      }
+      ep
     },
     "months" = {
       ixmon <- posixltindex$year * 100L + 190000L + posixltindex$mon
-      ep <- .Call("endpoints", ixmon, 1L, 1L, addlast, PACKAGE='xts')
-      if(k > 1)
-        ep[seq(1,length(ep),k)]
-      else ep
+      ep <- .Call(C_endpoints, ixmon, 1L, 1L, addlast)
+      if(k > 1) {
+        ep <- include_last(ep, k)
+      }
+      ep
     },
     "weeks" = {
-      .Call("endpoints", .index(x)+3L*86400L, 604800L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, .index(x)+3L*86400L, 604800L, k, addlast)
     },
     "days" = {
       ixyday <- posixltindex$year * 1000L + 1900000L + posixltindex$yday
-      .Call("endpoints", ixyday, 1L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, ixyday, 1L, k, addlast)
     },
     # non-date slicing should be indifferent to TZ and DST, so use math instead
     "hours" = {
-      .Call("endpoints", .index(x), 3600L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, .index(x), 3600L, k, addlast)
     },
     "minutes" = {
-      .Call("endpoints", .index(x), 60L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, .index(x), 60L, k, addlast)
     },
     "seconds" = {
-      .Call("endpoints", .index(x), 1L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, .index(x), 1L, k, addlast)
     },
     "ms" = ,
     "milliseconds" = {
       sec2ms <- .index(x) * 1e3
-      .Call("endpoints", sec2ms, 1L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, sec2ms, 1L, k, addlast)
     },
     "us" = ,
     "microseconds" = {
       sec2us <- .index(x) * 1e6
-      .Call("endpoints", sec2us, 1L, k, addlast, PACKAGE='xts')
+      .Call(C_endpoints, sec2us, 1L, k, addlast)
     }
   )
 }
