@@ -62,7 +62,7 @@ function(x, ...)
 function(x, value)
 {
   if (!is.null(value)) {
-    tzone <- as.character(value)
+    value <- as.character(value)
   }
   attr(x, "tzone") <- value
   x
@@ -75,7 +75,9 @@ function(x, ...)
 
   # For xts objects created pre-0.10.3
   if (is.null(tzone)) {
-    warning("index does not have a ", sQuote("tzone"), " attribute")
+    if (isTRUE(getOption("xts.warn.index.missing.tzone", FALSE))) {
+      warning("index does not have a ", sQuote("tzone"), " attribute")
+    }
 
     tzone <- attr(x, "tzone")
     if (is.null(tzone)) {
@@ -109,25 +111,42 @@ isUTC <- function(tz = NULL) {
 
 check.TZ <- function(x, ...)
 {
-  #if( !getOption("xts_check_TZ", FALSE))
-  #  return()
   check <- getOption("xts_check_TZ")
-  if( !is.null(check) && !check)
+
+  if (!is.null(check) && !check) {
     return()
-  STZ <- as.character(Sys.getenv("TZ"))
-  if(any(tclass(x) %in% .classesWithoutTZ)) {
+  }
+
+  x_tz <- tzone(x)
+
+  if (any(tclass(x) %in% .classesWithoutTZ)) {
     # warn if tzone is not UTC or GMT (GMT is not technically correct, since
     # it *is* a timezone, but it should work for all practical purposes)
-    if (!isUTC(tzone(x)))
-      warning(paste0("index class is ", paste(class(index(x)), collapse=", "),
-        ", which does not support timezones.\nExpected 'UTC' timezone",
-        ", but tzone is ", sQuote(tzone(x))), call.=FALSE)
-    else
+    if (!isUTC(x_tz)) {
+      warning(paste0("object index class (", paste(tclass(x), collapse = ", "),
+        ") does not support timezones.\nExpected 'UTC' timezone, but tzone is ",
+        sQuote(x_tz)), call. = FALSE)
+    } else {
       return()
+    }
+  }
+
+  x_tz_str <- as.character(x_tz)
+  sys_tz <- Sys.getenv("TZ")
+
+  if (!is.null(tzone(x)) && x_tz_str != "" && !identical(sys_tz, x_tz_str)) {
+    msg <- paste0("object timezone (", x_tz, ") is different ",
+                  "from system timezone (", sys_tz, ")")
+
+    if (is.null(check)) {
+      # xts_check_TZ is NULL by default
+      # set to TRUE after messaging user how to disable the warning
+      msg <- paste0(msg, "\n  NOTE: set 'options(xts_check_TZ = FALSE)'",
+                    "to disable this warning\n",
+                    "    This note is displayed once per session")
+      options(xts_check_TZ = TRUE)
+    }
+
+    warning(msg, call. = FALSE)
   }
-  if(!is.null(tzone(x)) && tzone(x) != "" &&
-     !identical(STZ, as.character(tzone(x))))
-    warning(paste("timezone of object (",tzone(x),
-                  ") is different than current timezone (",STZ,").",sep=""),
-           call.=FALSE)
 }
